// AP(r) Computer Science Marine Biology Simulation:
// The AbstractFishDisplay class is copyright(c) 2002 College Entrance
// Examination Board (www.collegeboard.com).
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.awt.Rectangle;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.BasicStroke;
import java.awt.Color;
import javax.swing.JOptionPane;

/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  This class provides common implementation code for drawing fish
 *  using the <code>LocatableDisplay</code> interface. It will translate, 
 *  scale, and rotate the graphics system as needed and then invoke its
 *  abstract <code>draw</code> method. Subclasses of this abstract class 
 *  define <code>draw</code> to just display a fish in a fixed size and
 *  orientation.
 *
 *  <p>
 *  The <code>AbstractFishDisplay</code> class is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Julie Zelenski
 *  @version 1 August 2002
 **/

public abstract class AbstractFishDisplay implements LocatableDisplay
{    
    
    /** Draw the given Fish.
     *  Subclasses should implement this method to draw the Fish facing North 
     *  in a cell of size (1,1) centered around (0,0) on the drawing surface.
     *  (All scaling/rotating has been done beforehand).
     *  @param   fish       fish we want to draw
     *  @param   comp       component on which to draw
     *  @param   g2         drawing surface
     **/
    abstract public void draw(Fish fish, Component comp, Graphics2D g2);
    
    
    /** Draw the given object.
     *  Scales and rotates the coordinate appropriately then invokes
     *  the simple draw method above that is only responsible for
     *  drawing a unit-length Fish facing North.
     *  (Precondition: <code>obj</code> is a Fish (or a Fish subclass).)
     *  @param   obj        object we want to draw
     *  @param   comp       component on which to draw
     *  @param   g2         drawing surface
     *  @param   rect       rectangle in which to draw 
     **/
    public void draw(Locatable obj, Component comp, Graphics2D g2, Rectangle rect)
    {
        Fish fish = (Fish)obj;
        float scaleFactor = Math.min(rect.width, rect.height);
        
        // Translate to center of fish
        g2.translate(rect.x + rect.width/2, rect.y + rect.height/2);
        
        // Rotate drawing surface before drawing to capture fish's
        // orientation (direction). 
        int rotationInDegrees = fish.direction().inDegrees();
        g2.rotate(Math.toRadians(rotationInDegrees));
        if (rotationInDegrees > 180) g2.scale(-1, 1); // flip side-to-side
    
        // Scale to size of rectangle, adjust stroke back to 1-pixel wide
        g2.scale(scaleFactor, scaleFactor);
        g2.setStroke(new BasicStroke(1.0f/scaleFactor));
    
        draw(fish, comp, g2);
    }
    
    /** Run a simple program to test a fish display object.
     *  The first command-line argument is expected to be the name of the
     *  fish display class to use (if no args, the class <code>FishDisplay</code> is used).  
     *  The default constructor is invoked to create an object of the display class 
     *  and that object is used to draw a fish in a dialog window.  This
     *  main may be useful when debugging a new fish display class.
     *  @param   args       array of command-line arguments, if non-empty,
     *                      first entry should be name of display class to test
     **/
    public static void main(String[] args) 
    throws Exception
    {
        if (args.length > 0)
        {
            Class displayCls = Class.forName(args[0]);
            LocatableDisplay displayObj = (LocatableDisplay)displayCls.newInstance();
            DisplayMap.associate("Fish", displayObj);
        }
        else
        {
            DisplayMap.associate("Fish", new FishDisplay());
        }

        JOptionPane.showMessageDialog(null, "", "Fish Display Test", 
                JOptionPane.INFORMATION_MESSAGE, 
                new FishToolbar.FishIcon(Fish.class, Color.red, 200, 200));
        System.exit(0);
    }
    
}
